/*  apcontrol.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: apcontrol.c,v 1.9 2002/05/15 19:16:24 sjah Exp $
 *
*/ 

#include <Python.h>
#include <control.h>
#include "pysession.h"

PyObject *moduleError;

/////////////////////////////// --------------------------------------
// DEFINITIONS
#define MAP_SIMPLE(NAME)						\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
									\
    if (!PyArg_ParseTuple (args, "i:"#NAME, &session))			\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

#define MAP_STR(NAME)							\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    char *s;								\
									\
    if (!PyArg_ParseTuple (args, "is:"#NAME, &session, &s))		\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, s);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

#define MAP_FLOAT(NAME)							\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    float f;								\
									\
    if (!PyArg_ParseTuple (args, "if:"#NAME, &session, &f))		\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, f);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

#define MAP_RET_FLOAT(NAME)						\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    float f;								\
									\
    if (!PyArg_ParseTuple (args, "i:"#NAME, &session))			\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, &f);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    return Py_BuildValue ("f", f);					\
};

#define MAP_RET_STRING(NAME, LEN)					\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    char buf[LEN];							\
									\
    if (!PyArg_ParseTuple (args, "i:"#NAME, &session))			\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, buf);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    return Py_BuildValue ("s", buf);					\
};

#define MAP_RET_INT(NAME)						\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    int i;								\
									\
    if (!PyArg_ParseTuple (args, "i:"#NAME, &session))			\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, &i);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    return Py_BuildValue ("i", i);					\
};

#define MAP_INT(NAME)							\
static PyObject *py##NAME (PyObject *self, PyObject *args)		\
{									\
    int session, rc;							\
    int i;								\
									\
    if (!PyArg_ParseTuple (args, "ii:"#NAME, &session, &i))		\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
	rc = ap_##NAME(session, i);					\
    Py_END_ALLOW_THREADS						\
									\
    if (!rc) {								\
	PyErr_SetString (moduleError, "error in "#NAME);		\
	return NULL;							\
    }									\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

//////////////////////////////////////////////////////////////////////
MAP_SIMPLE (play)
MAP_SIMPLE (stop)
MAP_SIMPLE (pause)
MAP_SIMPLE (unpause)
MAP_SIMPLE (next)
MAP_SIMPLE (prev)
MAP_SIMPLE (ping)
MAP_SIMPLE (quit)
MAP_SIMPLE (clear_playlist)
MAP_STR (add_path)
MAP_FLOAT (set_speed)
MAP_RET_FLOAT (get_speed)
MAP_RET_STRING (get_title, AP_TITLE_MAX)
MAP_RET_STRING (get_album, AP_ALBUM_MAX)
MAP_RET_STRING (get_genre, AP_GENRE_MAX)
MAP_RET_STRING (get_artist, AP_ARTIST_MAX)
MAP_RET_STRING (get_session_name, AP_SESSION_MAX)
MAP_RET_STRING (get_status, AP_STATUS_MAX)
MAP_RET_STRING (get_stream_type, AP_STREAM_TYPE_MAX)
MAP_RET_INT (get_position)
MAP_RET_INT (get_length)
MAP_RET_INT (get_frame)
MAP_RET_INT (get_frames)
MAP_INT (set_position)
MAP_INT (set_position_relative)
MAP_INT (set_frame)
    
// ------------------------------------------------------------------
// ap_version
static PyObject *pyversion (PyObject *self, PyObject *args)
{
    int rc;
    
    if (!PyArg_ParseTuple (args, ":version"))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
	rc = ap_version ();
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("i", rc);
}

// ------------------------------------------------------------------
// ap_session_running
static PyObject *pysession_running (PyObject *self, PyObject *args)
{
    int rc, session;
    
    if (!PyArg_ParseTuple (args, "i:session_running", &session))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
	rc = ap_session_running (session);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("i", rc);
}

// ------------------------------------------------------------------
// ap_find_session
static PyObject *pyfind_session (PyObject *self, PyObject *args)
{
    int rc, session;
    char *name;
    
    if (!PyArg_ParseTuple (args, "s:find_session", &name))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
	rc = ap_find_session (name, &session);
    Py_END_ALLOW_THREADS

    if (!rc) {
	PyErr_SetString (moduleError, "There is no such session or error happen");
	return NULL;
    }
 
    return Py_BuildValue ("i", session);
}

// ------------------------------------------------------------------
// Constructor for Session object
static PyObject *pysession (PyObject *self, PyObject *args)
{
    int session = 0;
 
    if (!PyArg_ParseTuple (args, "|i:Session", &session))
	return NULL;

    return (PyObject*)PySession_New (session);
}

// - -- ---- - -- - - - -          - - - - -- - ------- - - - - - -
// METHODS
static PyMethodDef methods[] = {
    {"play", pyplay, METH_VARARGS},
    {"stop", pystop, METH_VARARGS},
    {"pause", pypause, METH_VARARGS},
    {"unpause", pyunpause, METH_VARARGS},
    {"prev", pyprev, METH_VARARGS},
    {"next", pynext, METH_VARARGS},
    {"ping", pyping, METH_VARARGS},
    {"quit", pyquit, METH_VARARGS},
    {"clear_playlist", pyclear_playlist, METH_VARARGS},
    {"add_path", pyadd_path, METH_VARARGS},
    {"set_speed", pyset_speed, METH_VARARGS},
    {"get_speed", pyget_speed, METH_VARARGS},
    {"get_title", pyget_title, METH_VARARGS},
    {"get_album", pyget_album, METH_VARARGS},
    {"get_genre", pyget_genre, METH_VARARGS},
    {"get_artist", pyget_artist, METH_VARARGS},
    {"get_session_name", pyget_session_name, METH_VARARGS},
    {"get_position", pyget_position, METH_VARARGS},
    {"get_length", pyget_length, METH_VARARGS},
    {"get_frame", pyget_frame, METH_VARARGS},
    {"get_frames", pyget_frames, METH_VARARGS},
    {"set_position", pyset_position, METH_VARARGS},
    {"set_position_relative", pyset_position_relative, METH_VARARGS},
    {"set_frame", pyset_frame, METH_VARARGS},
    {"get_status", pyget_status, METH_VARARGS},
    {"get_stream_type", pyget_stream_type, METH_VARARGS},
    {"version", pyversion, METH_VARARGS},
    {"find_session", pyfind_session, METH_VARARGS},
    {"session_running", pysession_running, METH_VARARGS},
    {"Session", pysession, METH_VARARGS},
    {NULL, NULL}
};

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
void initapcontrol ()
{    
    PyObject *m = Py_InitModule ("apcontrol", methods);

    // Error
    moduleError = PyErr_NewException ("apcontrol.Error", NULL, NULL);
    PyModule_AddObject (m, "Error", moduleError);
}
