# position.py
# Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#   $Id: position.py,v 1.1 2002/05/11 00:01:09 sjah Exp $
#
 
__author__ = "Evgeny Chukreev <codedj@echo.ru>"
__date__ = "$Date: 2002/05/11 00:01:09 $"
__version__ = "$Revision: 1.1 $"
__license__ = "GNU"

import alsaplayer, gtk, aptk.misc, GTK

# - - - - - - - - Scale --------------------------------------------------
class _Scale:
    def __init__ (self, cp):
	# Initialize adjustment
	adj = gtk.GtkAdjustment ()
	self.set_adjustment (adj)

	# Remember
	self.__cp = cp
	self.__adj = adj
	self.__dragging = 0

	# Do not show digits by default
	self.set_draw_value (0)

	# Start timeout which will update indicator state every 100ms
	self.__timeout = gtk.timeout_add (100, self.__cb_timeout)

	# Signals
	self.connect ("button_press_event", self.__press_event)
	self.connect ("button_release_event", self.__release_event)
	self.connect ("motion_notify_event", self.__motion_event)

	# Update
	self.__update_state ()

    def __update_state (self):
	"""Updates indicator state."""
	
	if self.__cp.can_seek ():
	    self.set_sensitive (GTK.TRUE)

	    if self.__dragging:
		value = self.__adj.value
	    else:
		value = self.__cp.get_position ()

	    # FIXME:
	    # Hack derived from Andy's gtk_interface.cpp. Why 34 is here? ;)
	    self.__adj.set_all (value = value,
				lower = 0,
				upper = self.__cp.get_frames () - 34,
				step_increment = 1,
				page_increment = 0,
				page_size = 0)
	else:
	    self.set_sensitive (GTK.FALSE)
	    self.__adj.set_all (value = 0,
				lower = 0,
				upper = 0,
				step_increment = 0,
				page_increment = 0,
				page_size = 0)

    def __cb_timeout (self):
	"""Update indicator state every 100ms."""
	
	self.__update_state ()
	return 1

    def __press_event (self, widget, event):
	"""Button press event handler."""

	self.__dragging = 1
	self.__motion = 0

    def __motion_event (self, widget, event):
	"""Motion notify event handler."""

	self.__motion = 1

    def __release_event (self, widget, event):
	"""Button release event handler."""

	if self.__motion:
	    self.__cp.seek (self.__adj.value)

	self.__dragging = 0

class HScale (gtk.GtkHScale, _Scale):
    def __init__ (self, cp):
	gtk.GtkHScale.__init__ (self)
	_Scale.__init__ (self, cp)


class VScale (gtk.GtkVScale, _Scale):
    def __init__ (self, cp):
	gtk.GtkVScale.__init__ (self)
	_Scale.__init__ (self, cp)
