/*  alsaplayermodule.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: alsaplayermodule.cpp,v 1.4 2002/04/27 01:12:09 sjah Exp $
 *
*/ 

#include "alsaplayermodule.h"
#include "Playlist.h"
#include "pyplayitem.h"
#include "pycoreplayer.h"
#include "pyprefs.h"

PyPlaylist_Object		*pyplaylist_default;
PyPrefs_Object			*pyap_prefs;

PyObject			*PyAlsaplayerError;
PyObject			*PyPlaylistBadError;
PyObject			*PyPlaylistDubiousError;

// ------------------------------------------------------------------
// Method of alsaplayer module which uses to retrieve playlist object
static PyObject *pyget_playlist (PyObject *self, PyObject *args)
{
    if (!PyArg_ParseTuple (args, ":get_playlist"))
	return NULL;

    Py_INCREF (pyplaylist_default);
    return (PyObject*)pyplaylist_default;
}

// ------------------------------------------------------------------
// Method of alsaplayer module which uses to retrieve global prefs object
static PyObject *pyget_prefs (PyObject *self, PyObject *args)
{
    if (!PyArg_ParseTuple (args, ":get_prefs"))
	return NULL;

    Py_INCREF (pyap_prefs);
    return (PyObject*)pyap_prefs;
}

// ------------------------------------------------------------------
// Constructor of PlayItem object
static PyObject *pyplayitem (PyObject *self, PyObject *args)
{
    char *filename;
    PlayItem *playitem;
 
    if (!PyArg_ParseTuple (args, "s:PlayItem", &filename))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    playitem = new PlayItem (filename);
    Py_END_ALLOW_THREADS
    
    return (PyObject*)PyPlayItem_New (playitem, 1);
}

// ------------------------------------------------------------------
// Constructor of CorePlayer object
static PyObject *pycoreplayer (PyObject *self, PyObject *args)
{
    Playlist *pl;
    AlsaNode *node;
    CorePlayer *coreplayer;
    
    if (!PyArg_ParseTuple (args, ":CorePlayer"))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    pl = pyplaylist_default->cobject;
    node = pl->GetCorePlayer ()->GetNode ();
    coreplayer = new CorePlayer (node);
    Py_END_ALLOW_THREADS
    
    return (PyObject*)PyCorePlayer_New (coreplayer, 1);
}


// ------------------------------------------------------------------
// Constructor of Playlist object
static PyObject *pyplaylist (PyObject *self, PyObject *args)
{
    Playlist *playlist;
    AlsaNode *node;
    
    if (!PyArg_ParseTuple (args, ":Playlist"))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    node = pyplaylist_default->cobject->GetCorePlayer ()->GetNode ();
    playlist = new Playlist (node);
    Py_END_ALLOW_THREADS
    
    return (PyObject*)PyPlaylist_New (playlist, 1);
}

// ------------------------------------------------------------------
// Constructor for Prefs object
static PyObject *pyprefs (PyObject *self, PyObject *args)
{
    char *filename;
    prefs_handle_t *prefs;
    
    if (!PyArg_ParseTuple (args, "s:Prefs", &filename))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    prefs = prefs_load (filename);
    Py_END_ALLOW_THREADS

    if (prefs == NULL) {
	PyErr_SetString (PyAlsaplayerError, "error loading preferences");
	return NULL;
    }
    
    return (PyObject*)PyPrefs_New (prefs, 1);
}


// -------------------------------------------------------------------
// Methods of alsaplayer module
static PyMethodDef alsaplayer_methods[] = {
    {"get_playlist", pyget_playlist, METH_VARARGS},
    {"get_prefs", pyget_prefs, METH_VARARGS},
    {"PlayItem", pyplayitem, METH_VARARGS},
    {"CorePlayer", pycoreplayer, METH_VARARGS},
    {"Playlist", pyplaylist, METH_VARARGS},
    {"Prefs", pyprefs, METH_VARARGS},
    {NULL, NULL}
};

////////////////////////////////////////////////////////////////
// Public function
void alsaplayermodule_init (void)
{
    PyObject *m = Py_InitModule ("alsaplayer", alsaplayer_methods);
    
    // Create exceptions
    // Error
    PyAlsaplayerError = PyErr_NewException ("alsaplayer.Error",
					     NULL, NULL);
    PyModule_AddObject (m, "Error", PyAlsaplayerError);

    // PlaylistBadError
    PyPlaylistBadError = PyErr_NewException ("alsaplayer.PlaylistBadError",
					     NULL, NULL);
    PyModule_AddObject (m, "PlaylistBadError", PyPlaylistBadError);

    // PlaylistDubiousError
    PyPlaylistDubiousError = PyErr_NewException ("alsaplayer.PlaylistDubiousError",
					         NULL, NULL);
    PyModule_AddObject (m, "PlaylistDubiousError", PyPlaylistDubiousError); 

    // Create constants
    PyModule_AddIntConstant (m, "PLAYLIST_FORMAT_M3U", PL_FORMAT_M3U);
}
