/*  pycoreplayer.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: pycoreplayer.cpp,v 1.3 2002/04/23 20:17:56 sjah Exp $
 *
*/ 

#include "alsaplayermodule.h"
#include "pycoreplayer.h"
#include "pystreaminfo.h"
#include "pyplaylist.h"

#define OBJECT PyCorePlayer_Object
#include "pydefs.h"

// Methods:
// C++ name			Python name
// --------------------------------------------------
// GetNode
// GetPosition			get_position
// SetSpeed			set_speed
// GetSpeed			get_speed
// GetVolume			get_volume
// SetVolume			set_volume
// GetPan			get_pan
// SetPan			set_pan
// GetCurrentTime		get_current_time
// GetStreamInfo		get_stream_info
// GetFrames			get_frames
// GetSampleRate		get_samplerate
// GetChannels			get_channels
// GetFrameSize			get_frame_size
// GetPlayer
// Start			start
// Stop				stop
// Seek				seek
// Load				load
// Unload			unload
// CanSeek			can_seek
// IsActive			is_active
// IsPlaying			is_playing

// TYPE -----------------------------------------------------
staticforward PyObject *getattr (PyObject *self, char *name);
staticforward void dealloc (PyCorePlayer_Object *self);
staticforward int compare (PyCorePlayer_Object *self, PyCorePlayer_Object *o);

PyTypeObject PyCorePlayer_Type = {
    PyObject_HEAD_INIT (&PyType_Type)
    0,
    "CorePlayer",
    sizeof (PyCorePlayer_Object),
    0,
    (destructor) dealloc,
    0,
    (getattrfunc) getattr,
    0,
    (cmpfunc) compare,
    0,
    0,
    0,
    0,
    0,
};

// MAP by DEFINITIONS ---------------------------------------------------
mapsimple(Unload, unload)

mapsimple_with_intret (GetPosition,		    get_position)
mapsimple_with_intret (GetVolume,		    get_volume)
mapsimple_with_intret (GetPan,			    get_pan)
mapsimple_with_intret (GetFrames,		    get_frames)
mapsimple_with_intret (GetFrameSize,		    get_frame_size)
mapsimple_with_intret (GetSampleRate,		    get_samplerate)
mapsimple_with_intret (GetChannels,		    get_channels)
mapsimple_with_intret (CanSeek,			    can_seek)
mapsimple_with_intret (IsActive,		    is_active)
mapsimple_with_intret (IsPlaying,		    is_playing)

mapsimple_with_floatret (GetSpeed,		    get_speed)
 
// # # # # # # # # # # GetCurrentTime (frame) # # # # # # # # # # # # 
static PyObject *pyget_current_time (PyCorePlayer_Object *self, PyObject *args)
{
    long rc;
    int frame = -1;

    if (!PyArg_ParseTuple (args, "|i:get_current_time", &frame))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->GetCurrentTime (frame);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("l", rc);
};

// # # # # # # # # # # SetSpeed (speed) # # # # # # # # # # # # 
static PyObject *pyset_speed (PyCorePlayer_Object *self, PyObject *args)
{
    float speed;
    int rc;
    
    if (!PyArg_ParseTuple (args, "f:set_speed", &speed))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->SetSpeed (speed);
    Py_END_ALLOW_THREADS
    
    if (rc) {
	PyErr_SetString (PyAlsaplayerError, "error in set_speed");
	return NULL;
    }
	
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # SetVolume (pan) # # # # # # # # # # # # 
static PyObject *pyset_volume (PyCorePlayer_Object *self, PyObject *args)
{
    int volume;
    
    if (!PyArg_ParseTuple (args, "i:set_volume", &volume))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    self->cobject->SetVolume (volume);
    Py_END_ALLOW_THREADS
    
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # SetPan (pan) # # # # # # # # # # # # 
static PyObject *pyset_pan (PyCorePlayer_Object *self, PyObject *args)
{
    int pan;
    
    if (!PyArg_ParseTuple (args, "i:set_pan", &pan))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    self->cobject->SetPan (pan);
    Py_END_ALLOW_THREADS
   
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Seek (pos) # # # # # # # # # # # # 
static PyObject *pyseek (PyCorePlayer_Object *self, PyObject *args)
{
    int pos, rc;
    
    if (!PyArg_ParseTuple (args, "i:seek", &pos))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->Seek (pos);
    Py_END_ALLOW_THREADS
    
    if (rc) {
	PyErr_SetString (PyAlsaplayerError, "error in seek");
	return NULL;
    }
	
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # GetStreamInfo () # # # # # # # # # # # # 
static PyObject *pyget_stream_info (PyCorePlayer_Object *self, PyObject *args)
{
    if (!PyArg_ParseTuple (args, ":get_stream_info"))
	return NULL;

    PyStreamInfo_Object *info = PyStreamInfo_New ();

    Py_BEGIN_ALLOW_THREADS
    self->cobject->GetStreamInfo (&info->cstruct);
    Py_END_ALLOW_THREADS
    
    return (PyObject*)info;
};

// # # # # # # # # # # Stop (streamer) # # # # # # # # # # # # 
static PyObject *pystop (PyCorePlayer_Object *self, PyObject *args)
{
    int streamer = 1;
    
    if (!PyArg_ParseTuple (args, "|i:stop", &streamer))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    self->cobject->Stop (streamer);
    Py_END_ALLOW_THREADS
    
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Start (reset) # # # # # # # # # # # # 
static PyObject *pystart (PyCorePlayer_Object *self, PyObject *args)
{
    int reset = 1, rc;
    
    if (!PyArg_ParseTuple (args, "|i:start", &reset))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->Start (reset);
    Py_END_ALLOW_THREADS
    
    if (!rc) {
	PyErr_SetString (PyAlsaplayerError, "error in start");
	return NULL;
    }
	
    Py_INCREF (Py_None);
    return Py_None;
};

// # # # # # # # # # # Load (path) # # # # # # # # # # # # # # #
static PyObject *pyload (PyCorePlayer_Object *self, PyObject *args)
{
    char *s;
    int rc;
 
    if (!PyArg_ParseTuple (args, "s:load", &s))
	return NULL;
    
    Py_BEGIN_ALLOW_THREADS
    rc = self->cobject->Load (s);
    Py_END_ALLOW_THREADS
	
    if (!rc) {
	PyErr_SetString (PyAlsaplayerError, "error in load");
	return NULL;
    }
    
    Py_INCREF (Py_None);
    return Py_None;
};


// ##############################################################
// ##############################################################
static PyMethodDef methods[] = {
    {"get_stream_info", (PyCFunction) pyget_stream_info, METH_VARARGS},
    {"get_position", (PyCFunction) pyget_position, METH_VARARGS},
    {"get_volume", (PyCFunction) pyget_volume, METH_VARARGS},
    {"set_volume", (PyCFunction) pyset_volume, METH_VARARGS},
    {"get_pan", (PyCFunction) pyget_pan, METH_VARARGS},
    {"set_pan", (PyCFunction) pyset_pan, METH_VARARGS},
    {"get_frames", (PyCFunction) pyget_frames, METH_VARARGS},
    {"get_frame_size", (PyCFunction) pyget_frame_size, METH_VARARGS},
    {"get_samplerate", (PyCFunction) pyget_samplerate, METH_VARARGS},
    {"get_channels", (PyCFunction) pyget_channels, METH_VARARGS},
    {"get_speed", (PyCFunction) pyget_speed, METH_VARARGS},
    {"set_speed", (PyCFunction) pyset_speed, METH_VARARGS},
    {"get_current_time", (PyCFunction) pyget_current_time, METH_VARARGS},
    {"can_seek", (PyCFunction) pycan_seek, METH_VARARGS},
    {"seek", (PyCFunction) pyseek, METH_VARARGS},
    {"load", (PyCFunction) pyload, METH_VARARGS},
    {"unload", (PyCFunction) pyunload, METH_VARARGS},
    {"start", (PyCFunction) pystart, METH_VARARGS},
    {"stop", (PyCFunction) pystop, METH_VARARGS},
    {"is_active", (PyCFunction) pyis_active, METH_VARARGS},
    {"is_playing", (PyCFunction) pyis_playing, METH_VARARGS},
    {"stop", (PyCFunction) pystop, METH_VARARGS},
    {NULL, NULL}
};

// ///////////////////////////////////////////////////////////////////
static PyObject *getattr (PyObject *self, char *name)
{   
    // METHOD
    return Py_FindMethod (methods, self, name);
}

// DESTRUCTOR -- - -- -- ---- - - - - -- - - - - - -
static void dealloc (PyCorePlayer_Object *self)
{
    if (self->delondealloc) {
	Py_BEGIN_ALLOW_THREADS
	delete self->cobject;
	Py_END_ALLOW_THREADS
    }
 
    if (self->reference_playlist)  Py_DECREF (self->reference_playlist);
   
    PyObject_Del (self);
}

// COMPARATOR -- - -- -- ---- - - - - -- - - - - - -
static int compare (PyCorePlayer_Object *self, PyCorePlayer_Object *o)
{
    return o->cobject == self->cobject ? 0 :
	   o->cobject >  self->cobject ? 1 : -1;
}

// Object creation function -  - - - - - - - - - - - - - - - - - - -
PyCorePlayer_Object *PyCorePlayer_New (CorePlayer *coreplayer, int delondealloc,
				       PyPlaylist_Object *reference_pl)
{
    PyCorePlayer_Object *pycoreplayer;

    pycoreplayer = PyObject_New (PyCorePlayer_Object, &PyCorePlayer_Type);
    pycoreplayer->cobject = coreplayer;
    pycoreplayer->delondealloc = delondealloc;
    pycoreplayer->reference_playlist = reference_pl;

    if (reference_pl)  Py_INCREF (reference_pl);

    return pycoreplayer;
}
