#ifndef __PYDEFS_H__
#define __PYDEFS_H__

extern PyThreadState *pycallback_tstate;

// ----------------------------------------------------------------------
// Pattern to map simple C command without args to python ones
#define mapsimple(CNAME, PYNAME)					\
static PyObject *py##PYNAME (PyObject *self, PyObject *args)		\
{									\
    if (!PyArg_ParseTuple (args, ":"#PYNAME))				\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
    ((OBJECT*)self)->cobject->##CNAME ();				\
    Py_END_ALLOW_THREADS						\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

// Pattern to map simple C command with locking arg to python ones
#define mapsimple_with_locking(CNAME, PYNAME)				\
static PyObject *py##PYNAME (PyObject *self, PyObject *args)	\
{									\
    int locking = 0;							\
									\
    if (!PyArg_ParseTuple (args, "|i:"#PYNAME, &locking))		\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
    ((OBJECT*)self)->cobject->##CNAME (locking);			\
    Py_END_ALLOW_THREADS						\
									\
    Py_INCREF (Py_None);						\
    return Py_None;							\
};

// Pattern to map simple C command with int result returning to python ones
#define mapsimple_with_intret(CNAME, PYNAME)				\
static PyObject *py##PYNAME (PyObject *self, PyObject *args)		\
{									\
    int rc;								\
									\
    if (!PyArg_ParseTuple (args, ":"#PYNAME))				\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
    rc = ((OBJECT*)self)->cobject->##CNAME ();				\
    Py_END_ALLOW_THREADS						\
									\
    return Py_BuildValue ("i", rc);					\
};

// Pattern to map simple C command with long result returning to python ones
#define mapsimple_with_longret(CNAME, PYNAME)				\
static PyObject *py##PYNAME (PyObject *self, PyObject *args)		\
{									\
    long rc;								\
									\
    if (!PyArg_ParseTuple (args, ":"#PYNAME))				\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
    rc = ((OBJECT*)self)->cobject->##CNAME ();				\
    Py_END_ALLOW_THREADS						\
									\
    return Py_BuildValue ("l", rc);					\
};

// Pattern to map simple C command with float result returning to python ones
#define mapsimple_with_floatret(CNAME, PYNAME)				\
static PyObject *py##PYNAME (PyObject *self, PyObject *args)		\
{									\
    float rc;								\
									\
    if (!PyArg_ParseTuple (args, ":"#PYNAME))				\
	return NULL;							\
									\
    Py_BEGIN_ALLOW_THREADS						\
    rc = ((OBJECT*)self)->cobject->##CNAME ();				\
    Py_END_ALLOW_THREADS						\
									\
    return Py_BuildValue ("f", rc);					\
};

#endif
