/*  pyplayitem.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: pyplayitem.cpp,v 1.5 2002/05/10 00:46:21 sjah Exp $
 *
*/ 

#include <string.h>
#include "pyplayitem.h"

staticforward PyObject *getattr (PyPlayItem_Object *self, char *name);
staticforward void dealloc (PyPlayItem_Object *self);

PyTypeObject PyPlayItem_Type = {
    PyObject_HEAD_INIT (&PyType_Type)
    0,
    "PlayItem",
    sizeof (PyPlayItem_Object),
    0,
    (destructor) dealloc,
    0,
    (getattrfunc) getattr,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
};

// ##############################################################
// ##############################################################
static PyMethodDef methods[] = {
    {NULL, NULL}
};

// ///////////////////////////////////////////////////////////////////
static PyObject *getattr (PyPlayItem_Object *self, char *name)
{   
    // MEMBERS
    if (!strcmp (name, "__members__"))
	return Py_BuildValue ("(s,s,s,s,s,s,s)", "filename", "title",
			      "artist", "playtime", "parsed", "album", "genre");
 
    PlayItem *playitem = self->cobject;
    
    // -- filename member
    if (!strcmp (name, "filename"))
	return Py_BuildValue ("s", playitem->filename.c_str ());

    // -- title member
    if (!strcmp (name, "title"))
	return Py_BuildValue ("s", playitem->title.c_str ());

    // -- artist member
    if (!strcmp (name, "artist"))
	return Py_BuildValue ("s", playitem->artist.c_str ());

    // -- album member
    if (!strcmp (name, "album"))
	return Py_BuildValue ("s", playitem->album.c_str ());

    // -- genre member
    if (!strcmp (name, "genre"))
	return Py_BuildValue ("s", playitem->genre.c_str ());
   
    // -- playtime member
    if (!strcmp (name, "playtime"))
	return Py_BuildValue ("i", playitem->playtime);
 
    // -- parsed member
    if (!strcmp (name, "parsed"))
	return Py_BuildValue ("i", playitem->Parsed());
    
    // METHOD
    return Py_FindMethod (methods, (PyObject*)self, name);
}

// DESTRUCTOR -- - -- -- ---- - - - - -- - - - - - -
static void dealloc (PyPlayItem_Object *self)
{   
    if (self->delondealloc) {
	Py_BEGIN_ALLOW_THREADS
	delete self->cobject;
	Py_END_ALLOW_THREADS
    }
    
    PyObject_Del (self);  
}

// Object creation function -  - - - - - - - - - - - - - - - - - - -
PyPlayItem_Object *PyPlayItem_New (PlayItem *playitem, int delondealloc)
{
    PyPlayItem_Object *pyplayitem;

    pyplayitem = PyObject_New (PyPlayItem_Object, &PyPlayItem_Type);
    pyplayitem->cobject = playitem;
    pyplayitem->delondealloc = delondealloc;

    return pyplayitem;
}
