/*  pyprefs.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: pyprefs.cpp,v 1.1 2002/04/27 01:12:09 sjah Exp $
 *
*/ 

#include <string.h>
#include "pyprefs.h"
#include "alsaplayermodule.h"

staticforward PyObject *getattr (PyPrefs_Object *self, char *name);
staticforward void dealloc (PyPrefs_Object *self);

// --------------------------------- TYPE ------------------------------
PyTypeObject PyPrefs_Type = {
    PyObject_HEAD_INIT (&PyType_Type)
    0,
    "Prefs",
    sizeof (PyPrefs_Object),
    0,
    (destructor) dealloc,	/* tp_dealloc */
    0,				/* tp_print */
    (getattrfunc) getattr,	/* tp_getattr */
    0,				/* tp_setattr */
    0,				/* tp_compare */
    0,				/* tp_repr */
    0,				/* tp_as_number */
    0,				/* tp_as_squence */
    0,				/* tp_as_mapping */
    0,				/* tp_hash */
};

// ##############################################################
static PyObject *get_string (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key, *val, *rc;
    
    if (!PyArg_ParseTuple (args, "sss:get_string", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = prefs_get_string (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("s", rc);
}


// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *get_int (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    int val, rc;
    
    if (!PyArg_ParseTuple (args, "ssi:get_int", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = prefs_get_int (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("i", rc);
}

// -- - - -- -- -   --- - - - - - - - - -  - - - - -- - - - -    --
static PyObject *get_float (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    float val, rc;
    
    if (!PyArg_ParseTuple (args, "ssf:get_float", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = prefs_get_float (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("f", rc);
}

// - ----- - - - - - - - - -- --- -  - --- -   -  - -   - - - - --
static PyObject *get_bool (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    int val, rc;
    
    if (!PyArg_ParseTuple (args, "ssi:get_bool", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = prefs_get_bool (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    return Py_BuildValue ("i", rc);
}

// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *set_int (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    int val;
    
    if (!PyArg_ParseTuple (args, "ssi:set_int", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    prefs_set_int (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    Py_INCREF (Py_None);
    return Py_None;
}

// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *set_string (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    char *val;
    
    if (!PyArg_ParseTuple (args, "sss:set_string", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    prefs_set_string (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    Py_INCREF (Py_None);
    return Py_None;
}

// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *set_float (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    float val;
    
    if (!PyArg_ParseTuple (args, "ssf:set_float", &section, &key, &val))
	return NULL;
    
    Py_BEGIN_ALLOW_THREADS
    prefs_set_float (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    Py_INCREF (Py_None);
    return Py_None;
}

// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *set_bool (PyPrefs_Object *self, PyObject *args)
{
    char *section, *key;
    int val;
    
    if (!PyArg_ParseTuple (args, "ssi:set_bool", &section, &key, &val))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    prefs_set_bool (self->cobject, section, key, val);
    Py_END_ALLOW_THREADS

    Py_INCREF (Py_None);
    return Py_None;
}

// - -  - - - - - - - - --    - -  - -  -   - - - - -   - - - -
static PyObject *save (PyPrefs_Object *self, PyObject *args)
{
    int rc;
    
    if (!PyArg_ParseTuple (args, ":save"))
	return NULL;

    Py_BEGIN_ALLOW_THREADS
    rc = prefs_save (self->cobject);
    Py_END_ALLOW_THREADS

    if (rc) {
	PyErr_SetString (PyAlsaplayerError, "error saving preferences");
	return NULL;
    }
	
    Py_INCREF (Py_None);
    return Py_None;
}

// ##############################################################
static PyMethodDef methods[] = {
    {"get_int", (PyCFunction)get_int, METH_VARARGS, 0},
    {"get_string", (PyCFunction)get_string, METH_VARARGS, 0},
    {"get_float", (PyCFunction)get_float, METH_VARARGS, 0},
    {"get_bool", (PyCFunction)get_bool, METH_VARARGS, 0},
    {"set_int", (PyCFunction)set_int, METH_VARARGS, 0},
    {"set_string", (PyCFunction)set_string, METH_VARARGS, 0},
    {"set_float", (PyCFunction)set_float, METH_VARARGS, 0},
    {"set_bool", (PyCFunction)set_bool, METH_VARARGS, 0},
    {"save", (PyCFunction)save, METH_VARARGS, 0},
    {NULL, NULL}
};

// ///////////////////////////////////////////////////////////////////
static PyObject *getattr (PyPrefs_Object *self, char *name)
{   
    return Py_FindMethod (methods, (PyObject*)self, name);
}

// DESTRUCTOR -- - -- -- ---- - - - - -- - - - - - -
static void dealloc (PyPrefs_Object *self)
{   
    if (self->delondealloc) {
	Py_BEGIN_ALLOW_THREADS
	prefs_free (self->cobject);
	Py_END_ALLOW_THREADS
    }
 
    PyObject_Del (self);  
}

// Object creation function -  - - - - - - - - - - - - - - - - - - -
PyPrefs_Object *PyPrefs_New (prefs_handle_t *prefs, int delondealloc)
{
    PyPrefs_Object *pyprefs;

    pyprefs = PyObject_New (PyPrefs_Object, &PyPrefs_Type);
    pyprefs->cobject = prefs;
    pyprefs->delondealloc = delondealloc;

    return pyprefs;
}
