/*  python.cpp
 *  Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  $Id: python.cpp,v 1.5 2002/05/03 16:49:10 sjah Exp $
 *
*/ 

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <errno.h>
#include <unistd.h>
#include <string.h>

#include <Python.h>

#include "AlsaPlayer.h"
#include "interface_plugin.h"
#include "Playlist.h"
#include "alsaplayer_error.h"

#include "alsaplayermodule.h"
#include "pyplaylist.h"
#include "pyprefs.h"
#include "pydefs.h"

PyThreadState	*pycallback_tstate = NULL;

int interface_python_init()
{
    return 1;
}

int interface_python_running()
{
    return 1;
}


int interface_python_stop()
{  
    return 1;
}

void interface_python_close()
{
    return;
}


int interface_python_start(Playlist *playlist, int argc, char **argv)
{     
    // Initialize Python
    Py_SetProgramName (argv[0]);
    Py_Initialize ();
    PySys_SetArgv (argc, argv);
    PyEval_InitThreads ();

    // Interpreter state ensure!
    PyThreadState *tstate = PyThreadState_Swap (NULL);
    pycallback_tstate = PyThreadState_New (tstate->interp);
    PyThreadState_Swap (tstate);

    // Create global objects
    pyplaylist_default = PyPlaylist_New (playlist);
    pyap_prefs = PyPrefs_New (ap_prefs);

    // Initialize module
    alsaplayermodule_init ();

    char *filename = global_interface_script;

    if (!filename) {
	// Input from stdin

	if (isatty (fileno (stdin))) {
	    // It is console
	    printf ("AlsaPlayer " ALSAPLAYER_VERSION "\n");
	    printf ("Python %s\n", Py_GetVersion ());
	    
	    PyObject *m = PyImport_ImportModule ("readline");

	    if (m == NULL)
		PyErr_Clear ();
	    else
		Py_DECREF (m);
	}
	
	PyRun_AnyFile (stdin, "<stdin>");
    } else {
	FILE *f = fopen (filename, "r");
	
	if (f) {
	    PyRun_AnyFile (f, filename);
	    fclose (f);
	} else {
	    alsaplayer_error ("python-interface: Couldn't open %s: %s",
		    filename, strerror (errno));
	}
    }
    
    // ------------------------------------------------------------------
    // SHUTDOWN PLUGIN
    
    // Decrement references to global objects
    Py_DECREF (pyplaylist_default);
    Py_DECREF (pyap_prefs);
    
    // Finalize Python
    Py_Finalize ();
   
    return 0;
}


interface_plugin default_plugin =
{
    INTERFACE_PLUGIN_VERSION,
    { "PYTHON interface v" VERSION },
    { "Evgeny Chukreev" },
    NULL,
    interface_python_init,
    interface_python_start,
    interface_python_running,
    interface_python_stop,
    interface_python_close
};

extern "C" {
    interface_plugin *interface_plugin_info()
    {
	return &default_plugin;
    }
}
